/*
Copyright (c) 2007, Sun Microsystems, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.
    * Neither the name of Sun Microsystems, Inc. nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.sun.me.web.request;

import com.rapidftr.utilities.Arrays;

public class Part {
    
    private final byte[] content;
    private final Arg[] headers;
    
    public Part(final byte[] data, final Arg[] headers) {

        // data is mandatory, everything else is optional
        if (data == null) {
            throw new IllegalArgumentException("part data must be supplied");
        }

        this.content = data;
        this.headers = headers;
    }
    
    public byte[] getData() {
        return content;
    }
    
    public Arg[] getHeaders() {
        return headers;
    }
    
    public String toString() {
    	
    	StringBuffer buffer = new StringBuffer();
    	for(int i=0;i<headers.length;i++)
    	{
    		buffer.append(headers[i]);
    		buffer.append("\n");
    	}
    	buffer.append(content);
    	return buffer.toString();
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Part part = (Part) o;

        if (!Arrays.equals(content, part.content)) return false;
        if (!Arrays.equals(headers, part.headers)) return false;

        return true;
    }

    public int hashCode() {
        int result = content != null ? Arrays.hashCode(content) : 0;
        result = 31 * result + (headers != null ? Arrays.hashCode(headers) : 0);
        return result;
    }
}
